# -*- coding: utf-8 -*-

"""
This module contains an implementation of our "heavy-tailed mutation operator,
along with a few utility functions and the classical (1+1) EA.
"""

__version__ = '1.0'

from fastga.operators import *

